#!/bin/sh
#----------------------------------------------------------------------
# /var/install/config.d/freeradius-update.sh - update or generate new foo
#                                              configuration
# Creation   : 2010-02-19 stm
# Last update: $Id: freeradius-update.sh 597 2013-01-27 20:48:21Z stm $
#
# Copyright (c) 2001-2009 the eisfair team, team(at)eisfair(dot)org
# Copyright (c) 2010 Stephan Manske, eisfair-develop(at)stephan(dot)manske-net(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------

#exec 2>/public/freeradius-update-trace$$.log
#set -x

# define this package
packages_name='freeradius'


COPYRIGHT="2010 Stephan Manske <eisfair-develop(at)stephan(dot)manske-net(dot)de>"

callDir=`pwd`

. /var/install/include/eislib

# include configlib for using printvar
. /var/install/include/configlib

### ---------------------------------------------------------------------------
### Set the default values for configuration
### ---------------------------------------------------------------------------

START_FREERADIUS='no'

FREERADIUS_CERT_COUNTRY='DE'
FREERADIUS_CERT_ORGNAME='myName EIS'              
FREERADIUS_CERT_MAIL='admin@mail.example'          
FREERADIUS_CERT_MAILME='yes'                      
FREERADIUS_CERT_CA_NAME='my Home Radius CA'              
FREERADIUS_CERT_SERVER_NAME='my Home Radius'      
FREERADIUS_CERT_CA_TIME='3650'                 
FREERADIUS_CERT_SERVER_TIME='365'                 
FREERADIUS_CERT_USER_TIME='750'                  

FREERADIUS_MAKE_NEW_CERTS_NOW='no'   

FREERADIUS_CLIENT_N='1'          

FREERADIUS_CLIENT_1_NAME='localhost'         
FREERADIUS_CLIENT_1_ACTIVE='yes'             
FREERADIUS_CLIENT_1_SECRET=''                
FREERADIUS_CLIENT_1_IP='127.0.0.1'           
FREERADIUS_CLIENT_1_NASTYPE='other'         

FREERADIUS_USER_N='1'                    

FREERADIUS_USER_1_NAME='name'            
FREERADIUS_USER_1_ACTIVE='no'            
FREERADIUS_USER_1_PASS=''                
FREERADIUS_USER_1_CERT_ONLY='no'              
FREERADIUS_USER_1_CERT_SERIAL=''
FREERADIUS_USER_1_MAIL='user@mail.example'
FREERADIUS_USER_1_MAILME='no'            
FREERADIUS_USER_1_CUSTOM_N='0' 
FREERADIUS_USER_1_CUSTOM_1='' 

FREERADIUS_REJECT_DELAY='2'            



add_variables()
{

    added=0
    mecho --info "Adding new parameter(s) ..." 

    # v0.3.5
    if ! grep -q ^FREERADIUS_USER_1_CUSTOM_N /etc/config.d/${packages_name}
    then
        added=1


        [ "$FREERADIUS_USER_N" = "" ] && FREERADIUS_USER_N=0      # FREERADIUS_USER_N must be set
     
        if [ $FREERADIUS_USER_N -eq 0 ]                     # print at least one example
        then
            FREERADIUS_USER_N=1
        fi
    
        # begin FREERADIUS_USER_N
        idx=1
    
        while [ "${idx}" -le "${FREERADIUS_USER_N}" ]
        do
    
            # section marker FREERADIUS_USER_N
            if [ "${idx}" -le  "${FREERADIUS_USER_N}"  ]
            then
                # CUSTOM_Parameters 
                eval FREERADIUS_USER_${idx}_CUSTOM_N='0'
                eval FREERADIUS_USER_${idx}_CUSTOM_1=''
                mecho --std "- FREERADIUS_USER_${idx}_CUSTOM_N='0'"
                mecho --std "- FREERADIUS_USER_${idx}_CUSTOM_1=''"
            fi
    
        # end FREERADIUS_CLIENT_N
        idx=`expr ${idx} + 1`
        done

    fi

    # v0.3.6
    if ! grep -q ^FREERADIUS_USER_1_CERT_SERIAL /etc/config.d/${packages_name}
    then
        added=1


        [ "$FREERADIUS_USER_N" = "" ] && FREERADIUS_USER_N=0      # FREERADIUS_USER_N must be set
     
        if [ $FREERADIUS_USER_N -eq 0 ]                     # print at least one example
        then
            FREERADIUS_USER_N=1
        fi
    
        # begin FREERADIUS_USER_N
        idx=1
    
        while [ "${idx}" -le "${FREERADIUS_USER_N}" ]
        do
    
            # section marker FREERADIUS_USER_N
            if [ "${idx}" -le  "${FREERADIUS_USER_N}"  ]
            then
                eval FREERADIUS_USER_${idx}_CERT_SERIAL=''
                mecho --std "- FREERADIUS_USER_${idx}_CERT_SERIAL='' (hidden)"
            fi
    
        # end FREERADIUS_USER_N
        idx=`expr ${idx} + 1`
        done

    fi


    if [ $added -eq 1 ]
    then
        mecho --warn "... read documentation for new parameter(s)!"
    else
        mecho --std "... nothing to do."
    fi


}

remove_variables()
{
    mecho --info "Remove old parameter(s) ..."
    mecho --std "... nothing to do."
}


modify_variables()
{

    modified=0
    mecho --info "modifying parameter(s) ..."

    # 0.3.2
    if [ "${FREERADIUS_CERT_CA_TIME_CA}" != ""  ]
    then
        FREERADIUS_CERT_CA_TIME=${FREERADIUS_CERT_CA_TIME_CA}
        mecho --std "- FREERADIUS_CERT_CA_TIME_CA is now FREERADIUS_CERT_CA_TIME"
        modified=1
    fi





    # 0.3.5
    if [ "${FREERADIUS_USER_1_CERT}" != ""  ]
    then
        modified=1

        [ "$FREERADIUS_USER_N" = "" ] && FREERADIUS_USER_N=0      # FREERADIUS_USER_N must be set
     
        if [ $FREERADIUS_USER_N -eq 0 ]                     # print at least one example
        then
            FREERADIUS_USER_N=1
        fi
    
        # begin FREERADIUS_USER_N
        idx=1
    
        while [ "${idx}" -le "${FREERADIUS_USER_N}" ]
        do
    
            # section marker FREERADIUS_USER_N
            if [ "${idx}" -le  "${FREERADIUS_USER_N}"  ]
            then
                # new variable name
                eval FREERADIUS_USER_${idx}_CERT_ONLY='$FREERADIUS_USER_'${idx}'_CERT'
                mecho --std "- FREERADIUS_USER_${idx}_CERT is now FREERADIUS_USER_${idx}_CERT_ONLY"
            fi
    
        # end FREERADIUS_CLIENT_N
        idx=`expr ${idx} + 1`
        done

    fi
    

    if [ $modified = 1 ]
    then
        mecho --warn "... read documentation for modified parameter(s)!"
    else
        mecho --std "... nothing to do."
    fi


}





### ---------------------------------------------------------------------------
### read old configuration and modify variables
### ---------------------------------------------------------------------------
modify_old_config()
{
    # read old values
    if [ -f /etc/config.d/${packages_name} ]
    then
        . /etc/config.d/${packages_name}
        
        mecho ""
                
        add_variables
        remove_variables
        modify_variables
        
    fi

}

### ---------------------------------------------------------------------------
### Write config and default files
### ---------------------------------------------------------------------------
make_config_file()
{
    internal_conf_file=${1}
    {
    #--------------------------------------------------------------------------------
    printgpl -conf ${packages_name} "2010-02-27" "stm" "${COPYRIGHT}"
    #--------------------------------------------------------------------------------

    #--------------------------------------------------------------------------------
    printgroup "Basic configuration"
    #--------------------------------------------------------------------------------

    printvar "START_FREERADIUS" "Start FreeRadius server / Use yes or no"

    #--------------------------------------------------------------------------------
    printcustomgroup 'Server certificate settings' << !EOC
    To generate CA and server certificates with the parameters below please make
    FREERADIUS_MAKE_NEW_CERTS_NOW='YES'.
!EOC
    #--------------------------------------------------------------------------------

    printvar "FREERADIUS_CERT_COUNTRY" "country code, eg. DE, FR, GB or US"
    printvar "FREERADIUS_CERT_ORGNAME" "Name of your organisation"
    printvar "FREERADIUS_CERT_MAIL" "contact mail address"
    printvar "FREERADIUS_CERT_MAILME" "mail new certificats?"
    printvar "FREERADIUS_CERT_CA_NAME" 
    printvar "FREERADIUS_CERT_SERVER_NAME"
    printvar "FREERADIUS_CERT_CA_TIME" "cert valid for x days"
    printvar "FREERADIUS_CERT_SERVER_TIME" "cert valid for x days"
    printvar "FREERADIUS_CERT_USER_TIME" "cert valid for x days"

    #--------------------------------------------------------------------------------
    printcustomgroup 'Make new Certificates / PKI' << !EOC
    WARNING: If you make this to "yes" a second time, all present certificates 
    will be deleted and users have to import the new ones! There will be a
    security question further on.
!EOC
    #--------------------------------------------------------------------------------

    printvar "FREERADIUS_MAKE_NEW_CERTS_NOW" "make new ca and server"
    printvar "" "certs NOW. Will delete"
    printvar "" "ALL actual certs! CA,"
    printvar "" "Server AND users!"

   #--------------------------------------------------------------------------------
    printcustomgroup 'radius client setting' << !EOC
    ATTENTION: FREERADIUS_CLIENT_%_NAME MUST be the name of a local host 
    or enter a FREERADIUS_CLIENT_%_IP! Otherwise FreeRADIUS will not start!
!EOC
    #--------------------------------------------------------------------------------


    [ "$FREERADIUS_CLIENT_N" = "" ] && FREERADIUS_CLIENT_N=0      # FREERADIUS_CLIENT_N must be set
 
    printvar "FREERADIUS_CLIENT_N" "Numer of clients (e.g. a wlan-ap)"

    if [ $FREERADIUS_CLIENT_N -eq 0 ]                     # print at least one example
    then
        FREERADIUS_CLIENT_N=1
    fi

    # begin FREERADIUS_CLIENT_N
    idx=1

    while [ "${idx}" -le "${FREERADIUS_CLIENT_N}" ]
    do

        # section marker FREERADIUS_CLIENT_N
        if [ "${idx}" -le  "${FREERADIUS_CLIENT_N}"  ]
        then
            #--------------------------------------------------------------------------------
#            printgroup "FREERADIUS_CLIENT_${idx}"
            printgroup "FREERADIUS_CLIENT_%i"
            #--------------------------------------------------------------------------------
            
            printvar "FREERADIUS_CLIENT_${idx}_NAME" "fully qualified hostname"
            printvar "FREERADIUS_CLIENT_${idx}_ACTIVE" "Use yes or no"
            printvar "FREERADIUS_CLIENT_${idx}_SECRET" "The RADIUS shared secret"
            printvar "" "used for communication between"
            printvar "" "the client/NAS and the RADIUS server."
            printvar "" "Should not be to short! (>15 characters)"
            printvar "FREERADIUS_CLIENT_${idx}_IP" "In case of dns failure"
            printvar "" "or if the client is configured"
            printvar "" "in dhcp server package"
            printvar "" "it should be better to"
            printvar "" "type the ip additionaly"
            printvar "FREERADIUS_CLIENT_${idx}_NASTYPE" "e.g. cisco (no"
            printvar "" "linksys!), computone,"
            printvar "" "livingston. In most"
            printvar "" "cases other."
        fi


    # end FREERADIUS_CLIENT_N
    idx=`expr ${idx} + 1`
    done


    #--------------------------------------------------------------------------------
    printgroup 'User settings'
    #--------------------------------------------------------------------------------

    [ "$FREERADIUS_USER_N" = "" ] && FREERADIUS_USER_N=0      # FREERADIUS_USER_N must be set
 
    printvar "FREERADIUS_USER_N" "Number of supplicants/users:"
    printvar "" "notebooks etc."

    if [ $FREERADIUS_USER_N -eq 0 ]                     # print at least one example
    then
        FREERADIUS_USER_N=1
    fi

    # begin FREERADIUS_USER_N
    idx=1

    while [ "${idx}" -le "${FREERADIUS_USER_N}" ]
    do

        # section marker FREERADIUS_USER_N
        if [ "${idx}" -le  "${FREERADIUS_USER_N}"  ]
        then
            #--------------------------------------------------------------------------------
#            printgroup "FREERADIUS_USER_${idx}"
            printgroup "FREERADIUS_USER_%i"
            #--------------------------------------------------------------------------------
           
            printvar "FREERADIUS_USER_${idx}_NAME" "User Name"
            printvar "FREERADIUS_USER_${idx}_ACTIVE" "Use: yes or no"
            printvar "FREERADIUS_USER_${idx}_PASS" "user password to connect"
            printvar "FREERADIUS_USER_${idx}_CERT_ONLY" "Use certificate instead of user/pass"
            printvar "FREERADIUS_USER_${idx}_CERT_SERIAL" "DO NOT CHANGE!"
            printvar "" "EXCEPTION: If username is modified or this is a whole new user:"
            printvar "" "In this case, _SERIAL must be cleared (by you)!"
            printvar "FREERADIUS_USER_${idx}_MAIL" "mail address"
            printvar "FREERADIUS_USER_${idx}_MAILME" "mail cert to user?"
            
            # CUSTOM_Parameters 

           [ '$FREERADIUS_USER_'${idx}'_CUSTOM_N' = "" ] && FREERADIUS_USER_${idx}_CUSTOM_N=0      # FREERADIUS_USER_${idx}_CUSTOM_N must be set
         
            printvar "FREERADIUS_USER_${idx}_CUSTOM_N" "Number of custom parameters"
            
            eval temp_CUSTOM_N='$FREERADIUS_USER_'${idx}'_CUSTOM_N'
        
            if [ ${temp_CUSTOM_N} -eq 0 ]                     # print at least one example
            then
                eval "temp_CUSTOM_N"="1"
            fi
        
            # begin FREERADIUS_USER_${idx}_CUSTOM_N
            idx2=1
        
            while [ "${idx2}" -le "${temp_CUSTOM_N}" ]
            do
        
                # section marker FREERADIUS_USER_${idx}_CUSTOM_N
                if [ "${idx2}" -le  "${temp_CUSTOM_N}"  ]
                then
                    printvar "FREERADIUS_USER_${idx}_CUSTOM_${idx2}" "Parameter"
                fi
     
        
            # end FREERADIUS_USER_${idx}_CUSTOM_N
            idx2=`expr ${idx2} + 1`
            done

       
              
        fi


    # end FREERADIUS_CLIENT_N
    idx=`expr ${idx} + 1`
    done


    #--------------------------------------------------------------------------------
    printgroup 'Extended configuration'
    #--------------------------------------------------------------------------------

    printvar "FREERADIUS_REJECT_DELAY" "x sec delay of a request reject"
    printvar "" "can slowdown DOS attack or brute"
    printvar "" "force attack"


    #--------------------------------------------------------------------------------
    printend
    #--------------------------------------------------------------------------------

    } > ${internal_conf_file}
    # Set rights
    chmod 0600 ${internal_conf_file}
    chown root ${internal_conf_file}
}

### -------------------------------------------------------------------------
### Create the check.d files
### -------------------------------------------------------------------------
make_check_file()
{

    printgpl -check ${packages_name} "2010-02-27" "stm" "${COPYRIGHT}" >/etc/check.d/${packages_name}
    cat >>/etc/check.d/${packages_name} <<EOF
# Variable                                  OPT_VARIABLE                          VARIABLE_N           VALUE
START_FREERADIUS                                -                                     -                    YESNO

FREERADIUS_REJECT_DELAY                      START_FREERADIUS                         -                    NUMERIC

FREERADIUS_CERT_COUNTRY                      START_FREERADIUS                         -                    NOBLANK
FREERADIUS_CERT_ORGNAME                      START_FREERADIUS                         -                    NOTEMPTY
FREERADIUS_CERT_MAIL                         START_FREERADIUS                         -                    MAILADDR
FREERADIUS_CERT_CA_NAME                      START_FREERADIUS                         -                    NOTEMPTY
FREERADIUS_CERT_SERVER_NAME                  START_FREERADIUS                         -                    NOTEMPTY
FREERADIUS_CERT_MAILME                       START_FREERADIUS                         -                    YESNO
FREERADIUS_CERT_CA_TIME                      START_FREERADIUS                         -                    NUMERIC
FREERADIUS_CERT_SERVER_TIME                  START_FREERADIUS                         -                    NUMERIC
FREERADIUS_CERT_USER_TIME                    START_FREERADIUS                         -                    NUMERIC
 
FREERADIUS_MAKE_NEW_CERTS_NOW                START_FREERADIUS                         -                    YESNO

FREERADIUS_CLIENT_N                          START_FREERADIUS                         -                    NUMERIC

FREERADIUS_CLIENT_%_NAME                     START_FREERADIUS                     FREERADIUS_CLIENT_N      DOMAIN
FREERADIUS_CLIENT_%_ACTIVE                   START_FREERADIUS                     FREERADIUS_CLIENT_N      YESNO
FREERADIUS_CLIENT_%_SECRET                   FREERADIUS_CLIENT_%_ACTIVE           FREERADIUS_CLIENT_N      PASSWD
FREERADIUS_CLIENT_%_SECRET                   FREERADIUS_CLIENT_%_ACTIVE           FREERADIUS_CLIENT_N      NOBLANK

FREERADIUS_CLIENT_%_IP                       FREERADIUS_CLIENT_%_ACTIVE           FREERADIUS_CLIENT_N      EIPADDR
FREERADIUS_CLIENT_%_NASTYPE                  FREERADIUS_CLIENT_%_ACTIVE           FREERADIUS_CLIENT_N      FREERADIUS_NAS_CUI

FREERADIUS_USER_N                           START_FREERADIUS                         -                    NUMERIC
 
FREERADIUS_USER_%_NAME                      START_FREERADIUS                    FREERADIUS_USER_N           NOTEMPTY
FREERADIUS_USER_%_ACTIVE                    START_FREERADIUS                    FREERADIUS_USER_N           YESNO
FREERADIUS_USER_%_PASS                      FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_N           PASSWD
FREERADIUS_USER_%_PASS                      FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_N           NOBLANK
FREERADIUS_USER_%_CERT_ONLY                 FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_N           YESNO
FREERADIUS_USER_%_MAIL                      FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_N           MAILADDR
FREERADIUS_USER_%_MAILME                    FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_N           YESNO
FREERADIUS_USER_%_CERT_SERIAL               FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_N           NONE
FREERADIUS_USER_%_CUSTOM_N                  FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_N           NUMERIC
FREERADIUS_USER_%_CUSTOM_%                  FREERADIUS_USER_%_ACTIVE            FREERADIUS_USER_%_CUSTOM_N  NOTEMPTY

EOF

    # Set rights for check.d file
    chmod 0600 /etc/check.d/${packages_name}
    chown root /etc/check.d/${packages_name}



    printgpl -check_exp ${packages_name} "2010-02-27" "stm" "${COPYRIGHT}" >/etc/check.d/${packages_name}.exp
    cat >>/etc/check.d/${packages_name}.exp <<EOF
#bla
FREERADIUS_NAS_CUI    = 'other|cisco|computone|livingston|max40xx|multitech|netserver|pathras|patton|portslave|tc|usrhiper'
                     : 'available nas types - default: other'
EOF

    # Set rights for check.exp file
    chmod 0600 /etc/check.d/${packages_name}.exp
    chown root /etc/check.d/${packages_name}.exp



   printgpl -check_ext ${packages_name} "2010-03-03" "stm" "${COPYRIGHT}" >/etc/check.d/${packages_name}.ext
   cat >>/etc/check.d/${packages_name}.ext <<EOF

foreach user in FREERADIUS_USER_%_NAME
do
        set user_count = 0
        foreach user2 in FREERADIUS_USER_%_NAME
        do
                if ( user == user2 )
                then
                        set user_count = user_count + 1
                fi

                if ( user_count == 2 )
                then
                        error "Username '\${user}' exists more then one time!"
                fi

        done
done

foreach client in FREERADIUS_CLIENT_%_NAME
do
        set count = 0
        foreach client2 in FREERADIUS_CLIENT_%_NAME
        do
                if ( client == client2 )
                then
                        set count = count + 1
                fi

                if ( count == 2 )
                then
                        error "Clientname '\${client}' exists more then one time!"
                fi

        done
done

if ("\${FREERADIUS_CERT_MAILME}" == "yes" )
then
    stat ("/var/install/packages/mail", test)
    if ("\$test_res" != "OK")
    then
        stat ("/var/install/packages/vmail", test)
        if ("\$test_res" != "OK")
        then
#            stat ("/var/install/packages/ssmtp", test)
#            if ("\$test_res" != "OK")
#            then
                error "A suitable mail package is required to mail certs! Please install mail or vmail or set FREERADIUS_CERT_MAILME='no' respectively FREERADIUS_USER_%_MAILME='no'. Error in configuration, aborting ..."
#            fi
        fi
    fi
fi


EOF

    # Set rights for check.ext file
    chmod 0600 /etc/check.d/${packages_name}.ext
    chown root /etc/check.d/${packages_name}.ext


}



### -------------------------------------------------------------------------
### Create the dialog.d files
### -------------------------------------------------------------------------

make_dialog_files()
{

    echo "#! /bin/sh" >/var/install/dialog.d/FREERADIUS_NAS_CUI.sh

#    printgpl -dialog ${packages_name} "2010-02-27" "stm" "${COPYRIGHT}" >>/var/install/dialog.d/FREERADIUS_NAS_CUI.sh
    printgpl -conf ${packages_name} "2010-02-27" "stm" "${COPYRIGHT}" >>/var/install/dialog.d/FREERADIUS_NAS_CUI.sh
    
    cat >>/var/install/dialog.d/FREERADIUS_NAS_CUI.sh <<EOF

. /var/install/include/cuilib
. /var/install/include/ecelib

#----------------------------------------------------------------------------
# exec_dailog
# ece --> request to create and execute dialog
#         $p2 --> main window handle
#         $p3 --> name of config variable
#----------------------------------------------------------------------------
exec_dialog()
{
    win="\${p2}"

    sellist="other,cisco,computone,livingston,max40xx,multitech,netserver,pathras,patton,portslave,tc,usrhiper"

    ece_select_list_dlg "\${win}" "NAS type" "\${sellist}"
}

#----------------------------------------------------------------------------
# main routine
#----------------------------------------------------------------------------

cui_init
cui_run

#----------------------------------------------------------------------------
# end
#----------------------------------------------------------------------------

exit 0

EOF

    # Set rights for check.ext file
    chmod 0755 /var/install/dialog.d/FREERADIUS_NAS_CUI.sh
    chown root /var/install/dialog.d/FREERADIUS_NAS_CUI.sh
}


### ---------------------------------------------------------------------------
### Main
### ---------------------------------------------------------------------------
# write default config file
make_config_file /etc/default.d/${packages_name}

# update from old version
modify_old_config

# write new config file
make_config_file /etc/config.d/${packages_name}

# write check.d file
make_check_file

#write dialog.d files
make_dialog_files

cd ${callDir}

exit 0
#========================================================================
# end
#========================================================================
